# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Optional

from bungio.models.base import BaseModel, HashObject, ManifestModel, custom_define, custom_field

if TYPE_CHECKING:
    from bungio.models import (
        DestinyEventCardDefinition,
        DestinyIconDefinition,
        DestinyItemFilterDefinition,
        DestinyPlugSetDefinition,
        DestinyPresentationNodeDefinition,
    )


@custom_define()
class DestinyDisplayPropertiesDefinition(BaseModel):
    """
    Many Destiny*Definition contracts - the "first order" entities of Destiny that have their own tables in the Manifest Database - also have displayable information. This is the base class for that display information.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        description: _No description given by bungie._
        has_icon: _No description given by bungie._
        high_res_icon: If this item has a high-res icon (at least for now, many things won't), then the path to that icon will be here.
        icon: Note that "icon" is sometimes misleading, and should be interpreted in the context of the entity. For instance, in Destiny 1 the DestinyRecordBookDefinition's icon was a big picture of a book. But usually, it will be a small square image that you can use as... well, an icon. They are currently represented as 96px x 96px images.
        icon_hash: _No description given by bungie._
        icon_sequences: _No description given by bungie._
        name: _No description given by bungie._
        manifest_icon_hash: Manifest information for `icon_hash`
    """

    description: str = custom_field()
    has_icon: bool = custom_field()
    high_res_icon: str = custom_field()
    icon: str = custom_field()
    icon_hash: int = custom_field()
    icon_sequences: list["DestinyIconSequenceDefinition"] = custom_field(
        metadata={"type": """list[DestinyIconSequenceDefinition]"""}
    )
    name: str = custom_field()
    manifest_icon_hash: Optional["DestinyIconDefinition"] = custom_field(default=None)


@custom_define()
class DestinyIconSequenceDefinition(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        frames: _No description given by bungie._
    """

    frames: list[str] = custom_field(metadata={"type": """list[str]"""})


@custom_define()
class DestinyPositionDefinition(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        x: _No description given by bungie._
        y: _No description given by bungie._
        z: _No description given by bungie._
    """

    x: int = custom_field()
    y: int = custom_field()
    z: int = custom_field()


@custom_define()
class DestinyGlobalConstantsDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        active_seals_root_node_hash: _No description given by bungie._
        active_triumphs_root_node_hash: _No description given by bungie._
        armor_archetype_plug_set_hash: _No description given by bungie._
        collection_badges_root_node_hash: _No description given by bungie._
        collections_root_node_hash: _No description given by bungie._
        crafting_root_node_hash: _No description given by bungie._
        destiny_reward_pass_rank_seal_images: _No description given by bungie._
        destiny_seasonal_hub_rank_icon_images: _No description given by bungie._
        exotic_catalysts_root_node_hash: _No description given by bungie._
        featured_items_list_hash: _No description given by bungie._
        guardian_ranks_root_node_hash: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        legacy_seals_root_node_hash: _No description given by bungie._
        legacy_triumphs_root_node_hash: _No description given by bungie._
        lore_root_node_hash: _No description given by bungie._
        medals_root_node_hash: _No description given by bungie._
        metrics_root_node_hash: _No description given by bungie._
        pathfinder_constants: Assorted constants for Pathfinder objectives
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        seasonal_hub_event_card_hash: _No description given by bungie._
        manifest_active_seals_root_node_hash: Manifest information for `active_seals_root_node_hash`
        manifest_active_triumphs_root_node_hash: Manifest information for `active_triumphs_root_node_hash`
        manifest_armor_archetype_plug_set_hash: Manifest information for `armor_archetype_plug_set_hash`
        manifest_collection_badges_root_node_hash: Manifest information for `collection_badges_root_node_hash`
        manifest_collections_root_node_hash: Manifest information for `collections_root_node_hash`
        manifest_crafting_root_node_hash: Manifest information for `crafting_root_node_hash`
        manifest_exotic_catalysts_root_node_hash: Manifest information for `exotic_catalysts_root_node_hash`
        manifest_featured_items_list_hash: Manifest information for `featured_items_list_hash`
        manifest_guardian_ranks_root_node_hash: Manifest information for `guardian_ranks_root_node_hash`
        manifest_legacy_seals_root_node_hash: Manifest information for `legacy_seals_root_node_hash`
        manifest_legacy_triumphs_root_node_hash: Manifest information for `legacy_triumphs_root_node_hash`
        manifest_lore_root_node_hash: Manifest information for `lore_root_node_hash`
        manifest_medals_root_node_hash: Manifest information for `medals_root_node_hash`
        manifest_metrics_root_node_hash: Manifest information for `metrics_root_node_hash`
        manifest_seasonal_hub_event_card_hash: Manifest information for `seasonal_hub_event_card_hash`
    """

    active_seals_root_node_hash: int = custom_field()
    active_triumphs_root_node_hash: int = custom_field()
    armor_archetype_plug_set_hash: int = custom_field()
    collection_badges_root_node_hash: int = custom_field()
    collections_root_node_hash: int = custom_field()
    crafting_root_node_hash: int = custom_field()
    destiny_reward_pass_rank_seal_images: "DestinyRewardPassRankSealImages" = custom_field()
    destiny_seasonal_hub_rank_icon_images: "DestinySeasonalHubRankIconImages" = custom_field()
    exotic_catalysts_root_node_hash: int = custom_field()
    featured_items_list_hash: int = custom_field()
    guardian_ranks_root_node_hash: int = custom_field()
    index: int = custom_field()
    legacy_seals_root_node_hash: int = custom_field()
    legacy_triumphs_root_node_hash: int = custom_field()
    lore_root_node_hash: int = custom_field()
    medals_root_node_hash: int = custom_field()
    metrics_root_node_hash: int = custom_field()
    pathfinder_constants: "DestinyPathfinderConstantsDefinition" = custom_field()
    redacted: bool = custom_field()
    seasonal_hub_event_card_hash: int = custom_field()
    manifest_active_seals_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_active_triumphs_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_armor_archetype_plug_set_hash: Optional["DestinyPlugSetDefinition"] = custom_field(default=None)
    manifest_collection_badges_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(
        default=None
    )
    manifest_collections_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_crafting_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_exotic_catalysts_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_featured_items_list_hash: Optional["DestinyItemFilterDefinition"] = custom_field(default=None)
    manifest_guardian_ranks_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_legacy_seals_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_legacy_triumphs_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_lore_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_medals_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_metrics_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_seasonal_hub_event_card_hash: Optional["DestinyEventCardDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPathfinderConstantsDefinition(BaseModel, HashObject):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        all_pathfinder_root_node_hashes: Root presentation nodes for all currently valid Pathfinder boards
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        pathfinder_topology: The topology of the Pathfinder board. The key is the index of the Record in the Pathfinder board, and the value is a list of the indices of Records that are connected to the Key Record. Using this topology, clients can ascertain if a Record can be unlocked, by checking if the objective of any connected Record has been completed and/or claimed.
        pathfinder_tree_tiers: The current shape of Pathfinder boards, where a Pathfinder board is stored as as flat list of Records. The key of this dictionary is the index at which a tier starts, and the value is the total number of objectives in the tier.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        the_pale_heart_pathfinder_root_node_hash: Pathfinder root node for The Pale Heart
        manifest_the_pale_heart_pathfinder_root_node_hash: Manifest information for `the_pale_heart_pathfinder_root_node_hash`
    """

    all_pathfinder_root_node_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    index: int = custom_field()
    pathfinder_topology: dict[int, list[int]] = custom_field(metadata={"type": """dict[int, list[int]]"""})
    pathfinder_tree_tiers: dict[int, int] = custom_field(metadata={"type": """dict[int, int]"""})
    redacted: bool = custom_field()
    the_pale_heart_pathfinder_root_node_hash: int = custom_field()
    manifest_the_pale_heart_pathfinder_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(
        default=None
    )


@custom_define()
class DestinyRewardPassRankSealImages(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        reward_pass_rank_seal_image_path: _No description given by bungie._
        reward_pass_rank_seal_premium_image_path: _No description given by bungie._
        reward_pass_rank_seal_premium_prestige_image_path: _No description given by bungie._
        reward_pass_rank_seal_prestige_image_path: _No description given by bungie._
    """

    reward_pass_rank_seal_image_path: str = custom_field()
    reward_pass_rank_seal_premium_image_path: str = custom_field()
    reward_pass_rank_seal_premium_prestige_image_path: str = custom_field()
    reward_pass_rank_seal_prestige_image_path: str = custom_field()


@custom_define()
class DestinySeasonalHubRankIconImages(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        seasonal_hub_rank_icon_active: _No description given by bungie._
        seasonal_hub_rank_icon_earning: _No description given by bungie._
        seasonal_hub_rank_icon_unearned: _No description given by bungie._
    """

    seasonal_hub_rank_icon_active: str = custom_field()
    seasonal_hub_rank_icon_earning: str = custom_field()
    seasonal_hub_rank_icon_unearned: str = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from datetime import datetime
from typing import TYPE_CHECKING, Union

from bungio.models.base import BaseEnum, BaseFlagEnum, BaseModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import UserInfoCard


class ApplicationScopes(BaseFlagEnum):
    """
    _No description given by bungie._
    """

    READ_BASIC_USER_PROFILE = 1
    """Read basic user profile information such as the user's handle, avatar icon, etc. """
    READ_GROUPS = 2
    """Read Group/Clan Forums, Wall, and Members for groups and clans that the user has joined. """
    WRITE_GROUPS = 4
    """Write Group/Clan Forums, Wall, and Members for groups and clans that the user has joined. """
    ADMIN_GROUPS = 8
    """Administer Group/Clan Forums, Wall, and Members for groups and clans that the user is a founder or an administrator. """
    BNET_WRITE = 16
    """Create new groups, clans, and forum posts, along with other actions that are reserved for Bungie.net elevated scope: not meant to be used by third party applications. """
    MOVE_EQUIP_DESTINY_ITEMS = 32
    """Move or equip Destiny items """
    READ_DESTINY_INVENTORY_AND_VAULT = 64
    """Read Destiny 1 Inventory and Vault contents. For Destiny 2, this scope is needed to read anything regarded as private. This is the only scope a Destiny 2 app needs for read operations against Destiny 2 data such as inventory, vault, currency, vendors, milestones, progression, etc. """
    READ_USER_DATA = 128
    """Read user data such as who they are web notifications, clan/group memberships, recent activity, muted users. """
    EDIT_USER_DATA = 256
    """Edit user data such as preferred language, status, motto, avatar selection and theme. """
    READ_DESTINY_VENDORS_AND_ADVISORS = 512
    """Access vendor and advisor data specific to a user. OBSOLETE. This scope is only used on the Destiny 1 API. """
    READ_AND_APPLY_TOKENS = 1024
    """Read offer history and claim and apply tokens for the user. """
    ADVANCED_WRITE_ACTIONS = 2048
    """Can perform actions that will result in a prompt to the user via the Destiny app. """
    PARTNER_OFFER_GRANT = 4096
    """Can use the partner offer api to claim rewards defined for a partner """
    DESTINY_UNLOCK_VALUE_QUERY = 8192
    """Allows an app to query sensitive information like unlock flags and values not available through normal methods. """
    USER_PII_READ = 16384
    """Allows an app to query sensitive user PII, most notably email information. """


@custom_define()
class ApiUsage(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        api_calls: Counts for on API calls made for the time range.
        throttled_requests: Instances of blocked requests or requests that crossed the warn threshold during the time range.
    """

    api_calls: list["Series"] = custom_field(metadata={"type": """list[Series]"""})
    throttled_requests: list["Series"] = custom_field(metadata={"type": """list[Series]"""})


@custom_define()
class Series(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        datapoints: Collection of samples with time and value.
        target: Target to which to datapoints apply.
    """

    datapoints: list["Datapoint"] = custom_field(metadata={"type": """list[Datapoint]"""})
    target: str = custom_field()


@custom_define()
class Datapoint(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        count: Count associated with timestamp
        time: Timestamp for the related count.
    """

    count: float = custom_field()
    time: datetime = custom_field()


@custom_define()
class Application(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        application_id: Unique ID assigned to the application
        application_type: _No description given by bungie._
        creation_date: Date the application was first added to our database.
        first_published: Date the first time the application status entered the 'Public' status.
        link: Link to website for the application where a user can learn more about the app.
        name: Name of the application
        origin: Value of the Origin header sent in requests generated by this application.
        override_authorize_view_name: An optional override for the Authorize view name.
        redirect_url: URL used to pass the user's authorization code to the application
        scope: Permissions the application needs to work
        status: Current status of the application.
        status_changed: Date the application status last changed.
        team: List of team members who manage this application on Bungie.net. Will always consist of at least the application owner.
    """

    application_id: int = custom_field()
    application_type: Union["OAuthApplicationType", int] = custom_field(
        converter=enum_converter("OAuthApplicationType")
    )
    creation_date: datetime = custom_field()
    first_published: datetime = custom_field()
    link: str = custom_field()
    name: str = custom_field()
    origin: str = custom_field()
    override_authorize_view_name: str = custom_field()
    redirect_url: str = custom_field()
    scope: int = custom_field(metadata={"int64": True})
    status: Union["ApplicationStatus", int] = custom_field(converter=enum_converter("ApplicationStatus"))
    status_changed: datetime = custom_field()
    team: list["ApplicationDeveloper"] = custom_field(metadata={"type": """list[ApplicationDeveloper]"""})


class OAuthApplicationType(BaseEnum):
    """
    _No description given by bungie._
    """

    NONE = 0
    """_No description given by bungie._ """
    CONFIDENTIAL = 1
    """Indicates the application is server based and can keep its secrets from end users and other potential snoops. """
    PUBLIC = 2
    """Indicates the application runs in a public place, and it can't be trusted to keep a secret. """


class ApplicationStatus(BaseEnum):
    """
    _No description given by bungie._
    """

    NONE = 0
    """No value assigned """
    PRIVATE = 1
    """Application exists and works but will not appear in any public catalog. New applications start in this state, test applications will remain in this state. """
    PUBLIC = 2
    """Active applications that can appear in an catalog. """
    DISABLED = 3
    """Application disabled by the owner. All authorizations will be treated as terminated while in this state. Owner can move back to private or public state. """
    BLOCKED = 4
    """Application has been blocked by Bungie. It cannot be transitioned out of this state by the owner. Authorizations are terminated when an application is in this state. """


@custom_define()
class ApplicationDeveloper(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        api_eula_version: _No description given by bungie._
        role: _No description given by bungie._
        user: _No description given by bungie._
    """

    api_eula_version: int = custom_field()
    role: Union["DeveloperRole", int] = custom_field(converter=enum_converter("DeveloperRole"))
    user: "UserInfoCard" = custom_field()


class DeveloperRole(BaseEnum):
    """
    _No description given by bungie._
    """

    NONE = 0
    """_No description given by bungie._ """
    OWNER = 1
    """_No description given by bungie._ """
    TEAM_MEMBER = 2
    """_No description given by bungie._ """



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from datetime import datetime
from typing import TYPE_CHECKING, Union

from bungio.models.base import BaseEnum, BaseFlagEnum, BaseModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import GeneralUser, GroupResponse, IgnoreResponse, PagedQuery, TagResponse


class ForumTopicsCategoryFiltersEnum(BaseFlagEnum):
    """
    _No description given by bungie._
    """

    NONE = 0
    """_No description given by bungie._ """
    LINKS = 1
    """_No description given by bungie._ """
    QUESTIONS = 2
    """_No description given by bungie._ """
    ANSWERED_QUESTIONS = 4
    """_No description given by bungie._ """
    MEDIA = 8
    """_No description given by bungie._ """
    TEXT_ONLY = 16
    """_No description given by bungie._ """
    ANNOUNCEMENT = 32
    """_No description given by bungie._ """
    BUNGIE_OFFICIAL = 64
    """_No description given by bungie._ """
    POLLS = 128
    """_No description given by bungie._ """


class ForumTopicsQuickDateEnum(BaseEnum):
    """
    _No description given by bungie._
    """

    ALL = 0
    """_No description given by bungie._ """
    LAST_YEAR = 1
    """_No description given by bungie._ """
    LAST_MONTH = 2
    """_No description given by bungie._ """
    LAST_WEEK = 3
    """_No description given by bungie._ """
    LAST_DAY = 4
    """_No description given by bungie._ """


class ForumTopicsSortEnum(BaseEnum):
    """
    _No description given by bungie._
    """

    DEFAULT = 0
    """_No description given by bungie._ """
    LAST_REPLIED = 1
    """_No description given by bungie._ """
    MOST_REPLIED = 2
    """_No description given by bungie._ """
    POPULARITY = 3
    """_No description given by bungie._ """
    CONTROVERSIALITY = 4
    """_No description given by bungie._ """
    LIKED = 5
    """_No description given by bungie._ """
    HIGHEST_RATED = 6
    """_No description given by bungie._ """
    MOST_UPVOTED = 7
    """_No description given by bungie._ """


@custom_define()
class PostResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        ignore_status: _No description given by bungie._
        is_active: _No description given by bungie._
        is_announcement: _No description given by bungie._
        is_pinned: _No description given by bungie._
        last_reply_timestamp: _No description given by bungie._
        latest_reply_author_id: _No description given by bungie._
        latest_reply_post_id: _No description given by bungie._
        locale: _No description given by bungie._
        popularity: _No description given by bungie._
        thumbnail: _No description given by bungie._
        url_media_type: _No description given by bungie._
        user_has_muted_post: _No description given by bungie._
        user_has_rated: _No description given by bungie._
        user_rating: _No description given by bungie._
    """

    ignore_status: "IgnoreResponse" = custom_field()
    is_active: bool = custom_field()
    is_announcement: bool = custom_field()
    is_pinned: bool = custom_field()
    last_reply_timestamp: datetime = custom_field()
    latest_reply_author_id: int = custom_field(metadata={"int64": True})
    latest_reply_post_id: int = custom_field(metadata={"int64": True})
    locale: str = custom_field()
    popularity: Union["ForumPostPopularity", int] = custom_field(converter=enum_converter("ForumPostPopularity"))
    thumbnail: str = custom_field()
    url_media_type: Union["ForumMediaType", int] = custom_field(converter=enum_converter("ForumMediaType"))
    user_has_muted_post: bool = custom_field()
    user_has_rated: bool = custom_field()
    user_rating: int = custom_field()


class ForumMediaType(BaseEnum):
    """
    _No description given by bungie._
    """

    NONE = 0
    """_No description given by bungie._ """
    IMAGE = 1
    """_No description given by bungie._ """
    VIDEO = 2
    """_No description given by bungie._ """
    YOUTUBE = 3
    """_No description given by bungie._ """


class ForumPostPopularity(BaseEnum):
    """
    _No description given by bungie._
    """

    EMPTY = 0
    """_No description given by bungie._ """
    DEFAULT = 1
    """_No description given by bungie._ """
    DISCUSSED = 2
    """_No description given by bungie._ """
    COOL_STORY = 3
    """_No description given by bungie._ """
    HEATING_UP = 4
    """_No description given by bungie._ """
    HOT = 5
    """_No description given by bungie._ """


@custom_define()
class PostSearchResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        authors: _No description given by bungie._
        available_pages: _No description given by bungie._
        groups: _No description given by bungie._
        has_more: _No description given by bungie._
        polls: _No description given by bungie._
        query: _No description given by bungie._
        recruitment_details: _No description given by bungie._
        related_posts: _No description given by bungie._
        replacement_continuation_token: _No description given by bungie._
        results: _No description given by bungie._
        searched_tags: _No description given by bungie._
        total_results: _No description given by bungie._
        use_total_results: If useTotalResults is true, then totalResults represents an accurate count. If False, it does not, and may be estimated/only the size of the current page. Either way, you should probably always only trust hasMore. This is a long-held historical throwback to when we used to do paging with known total results. Those queries toasted our database, and we were left to hastily alter our endpoints and create backward- compatible shims, of which useTotalResults is one.
    """

    authors: list["GeneralUser"] = custom_field(metadata={"type": """list[GeneralUser]"""})
    available_pages: int = custom_field()
    groups: list["GroupResponse"] = custom_field(metadata={"type": """list[GroupResponse]"""})
    has_more: bool = custom_field()
    polls: list["PollResponse"] = custom_field(metadata={"type": """list[PollResponse]"""})
    query: "PagedQuery" = custom_field()
    recruitment_details: list["ForumRecruitmentDetail"] = custom_field(
        metadata={"type": """list[ForumRecruitmentDetail]"""}
    )
    related_posts: list["PostResponse"] = custom_field(metadata={"type": """list[PostResponse]"""})
    replacement_continuation_token: str = custom_field()
    results: list["PostResponse"] = custom_field(metadata={"type": """list[PostResponse]"""})
    searched_tags: list["TagResponse"] = custom_field(metadata={"type": """list[TagResponse]"""})
    total_results: int = custom_field()
    use_total_results: bool = custom_field()


@custom_define()
class PollResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        results: _No description given by bungie._
        topic_id: _No description given by bungie._
        total_votes: _No description given by bungie._
    """

    results: list["PollResult"] = custom_field(metadata={"type": """list[PollResult]"""})
    topic_id: int = custom_field(metadata={"int64": True})
    total_votes: int = custom_field()


@custom_define()
class PollResult(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        answer_slot: _No description given by bungie._
        answer_text: _No description given by bungie._
        last_vote_date: _No description given by bungie._
        requesting_user_voted: _No description given by bungie._
        votes: _No description given by bungie._
    """

    answer_slot: int = custom_field()
    answer_text: str = custom_field()
    last_vote_date: datetime = custom_field()
    requesting_user_voted: bool = custom_field()
    votes: int = custom_field()


@custom_define()
class ForumRecruitmentDetail(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        approved: _No description given by bungie._
        conversation_id: _No description given by bungie._
        fireteam: _No description given by bungie._
        intensity: _No description given by bungie._
        kicked_player_ids: _No description given by bungie._
        microphone_required: _No description given by bungie._
        player_slots_remaining: _No description given by bungie._
        player_slots_total: _No description given by bungie._
        tone: _No description given by bungie._
        topic_id: _No description given by bungie._
    """

    approved: bool = custom_field()
    conversation_id: int = custom_field(metadata={"int64": True})
    fireteam: list["GeneralUser"] = custom_field(metadata={"type": """list[GeneralUser]"""})
    intensity: Union["ForumRecruitmentIntensityLabel", int] = custom_field(
        converter=enum_converter("ForumRecruitmentIntensityLabel")
    )
    kicked_player_ids: list[int] = custom_field(metadata={"type": """list[int]"""})
    microphone_required: bool = custom_field()
    player_slots_remaining: int = custom_field()
    player_slots_total: int = custom_field()
    tone: Union["ForumRecruitmentToneLabel", int] = custom_field(converter=enum_converter("ForumRecruitmentToneLabel"))
    topic_id: int = custom_field(metadata={"int64": True})


class ForumRecruitmentIntensityLabel(BaseEnum):
    """
    _No description given by bungie._
    """

    NONE = 0
    """_No description given by bungie._ """
    CASUAL = 1
    """_No description given by bungie._ """
    PROFESSIONAL = 2
    """_No description given by bungie._ """


class ForumRecruitmentToneLabel(BaseEnum):
    """
    _No description given by bungie._
    """

    NONE = 0
    """_No description given by bungie._ """
    FAMILY_FRIENDLY = 1
    """_No description given by bungie._ """
    ROWDY = 2
    """_No description given by bungie._ """


class ForumPostSortEnum(BaseEnum):
    """
    _No description given by bungie._
    """

    DEFAULT = 0
    """_No description given by bungie._ """
    OLDEST_FIRST = 1
    """_No description given by bungie._ """


class CommunityContentSortMode(BaseEnum):
    """
    _No description given by bungie._
    """

    TRENDING = 0
    """_No description given by bungie._ """
    LATEST = 1
    """_No description given by bungie._ """
    HIGHEST_RATED = 2
    """_No description given by bungie._ """



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING

from bungio.models.base import BaseModel, custom_define, custom_field

if TYPE_CHECKING:
    from bungio.models import DestinyObjectiveProgress


@custom_define()
class DestinyChallengeStatus(BaseModel):
    """
    Represents the status and other related information for a challenge that is - or was - available to a player.  A challenge is a bonus objective, generally tacked onto Quests or Activities, that provide additional variations on play.

    None
    Attributes:
        objective: The progress - including completion status - of the active challenge.
    """

    objective: "DestinyObjectiveProgress" = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING

from bungio.models.base import HashObject, ManifestModel, custom_define, custom_field

if TYPE_CHECKING:
    from bungio.models import DestinyDisplayPropertiesDefinition


@custom_define()
class DestinyActivityModifierDefinition(ManifestModel, HashObject):
    """
    Modifiers - in Destiny 1, these were referred to as "Skulls" - are changes that can be applied to an Activity.

    None
    Attributes:
        display_in_activity_selection: _No description given by bungie._
        display_in_nav_mode: _No description given by bungie._
        display_properties: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
    """

    display_in_activity_selection: bool = custom_field()
    display_in_nav_mode: bool = custom_field()
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    index: int = custom_field()
    redacted: bool = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Optional

from bungio.models.base import BaseModel, custom_define, custom_field

if TYPE_CHECKING:
    from bungio.models import DestinyInventoryItemDefinition, DyeReference


@custom_define()
class DestinyCharacterCustomization(BaseModel):
    """
    Raw data about the customization options chosen for a character's face and appearance. You can look up the relevant class/race/gender combo in DestinyCharacterCustomizationOptionDefinition for the character, and then look up these values within the CustomizationOptions found to pull some data about their choices. Warning: not all of that data is meaningful. Some data has useful icons. Others have nothing, and are only meant for 3D rendering purposes (which we sadly do not expose yet)

    None
    Attributes:
        decal_color: _No description given by bungie._
        decal_index: _No description given by bungie._
        eye_color: _No description given by bungie._
        face: _No description given by bungie._
        feature_colors: _No description given by bungie._
        feature_index: _No description given by bungie._
        hair_colors: _No description given by bungie._
        hair_index: _No description given by bungie._
        lip_color: _No description given by bungie._
        personality: _No description given by bungie._
        skin_color: _No description given by bungie._
        wear_helmet: _No description given by bungie._
    """

    decal_color: int = custom_field()
    decal_index: int = custom_field()
    eye_color: int = custom_field()
    face: int = custom_field()
    feature_colors: list[int] = custom_field(metadata={"type": """list[int]"""})
    feature_index: int = custom_field()
    hair_colors: list[int] = custom_field(metadata={"type": """list[int]"""})
    hair_index: int = custom_field()
    lip_color: int = custom_field()
    personality: int = custom_field()
    skin_color: int = custom_field()
    wear_helmet: bool = custom_field()


@custom_define()
class DestinyCharacterPeerView(BaseModel):
    """
    A minimal view of a character's equipped items, for the purpose of rendering a summary screen or showing the character in 3D.

    None
    Attributes:
        equipment: _No description given by bungie._
    """

    equipment: list["DestinyItemPeerView"] = custom_field(metadata={"type": """list[DestinyItemPeerView]"""})


@custom_define()
class DestinyItemPeerView(BaseModel):
    """
    Bare minimum summary information for an item, for the sake of 3D rendering the item.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        dyes: The list of dyes that have been applied to this item.
        item_hash: The hash identifier of the item in question. Use it to look up the DestinyInventoryItemDefinition of the item for static rendering data.
        manifest_item_hash: Manifest information for `item_hash`
    """

    dyes: list["DyeReference"] = custom_field(metadata={"type": """list[DyeReference]"""})
    item_hash: int = custom_field()
    manifest_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Union

from bungio.models.base import BaseModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import BungieCredentialType


@custom_define()
class GetCredentialTypesForAccountResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        credential_as_string: _No description given by bungie._
        credential_display_name: _No description given by bungie._
        credential_type: _No description given by bungie._
        is_public: _No description given by bungie._
    """

    credential_as_string: str = custom_field()
    credential_display_name: str = custom_field()
    credential_type: Union["BungieCredentialType", int] = custom_field(converter=enum_converter("BungieCredentialType"))
    is_public: bool = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Optional, Union

from bungio.models import (
    BungieMembershipType,
    DestinyFireteamFinderApplicationType,
    DestinyFireteamFinderApplyToListingResponse,
    DestinyFireteamFinderBulkGetListingStatusResponse,
    DestinyFireteamFinderGetApplicationResponse,
    DestinyFireteamFinderGetCharacterActivityAccessResponse,
    DestinyFireteamFinderGetListingApplicationsResponse,
    DestinyFireteamFinderGetLobbyOffersResponse,
    DestinyFireteamFinderGetPlayerApplicationsResponse,
    DestinyFireteamFinderGetPlayerLobbiesResponse,
    DestinyFireteamFinderGetPlayerOffersResponse,
    DestinyFireteamFinderHostLobbyRequest,
    DestinyFireteamFinderHostLobbyResponse,
    DestinyFireteamFinderJoinLobbyRequest,
    DestinyFireteamFinderKickPlayerRequest,
    DestinyFireteamFinderListing,
    DestinyFireteamFinderLobbyResponse,
    DestinyFireteamFinderOffer,
    DestinyFireteamFinderRespondToApplicationRequest,
    DestinyFireteamFinderRespondToApplicationResponse,
    DestinyFireteamFinderRespondToAuthenticationRequest,
    DestinyFireteamFinderRespondToAuthenticationResponse,
    DestinyFireteamFinderRespondToOfferRequest,
    DestinyFireteamFinderRespondToOfferResponse,
    DestinyFireteamFinderSearchListingsByClanRequest,
    DestinyFireteamFinderSearchListingsByClanResponse,
    DestinyFireteamFinderSearchListingsByFiltersRequest,
    DestinyFireteamFinderSearchListingsByFiltersResponse,
    DestinyFireteamFinderUpdateLobbySettingsRequest,
    DestinyFireteamFinderUpdateLobbySettingsResponse,
)
from bungio.models.auth import AuthData
from bungio.models.base import ClientMixin, custom_define


@custom_define()
class FireteamFinderRouteInterface(ClientMixin):
    async def activate_lobby(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        force_activation: Optional[bool] = None,
        auth: Optional[AuthData] = None,
    ) -> bool:
        """
        Activates a lobby and initializes it as an active Fireteam.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to activate.
            force_activation: Optional boolean to forcibly activate the lobby, kicking pending applicants.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.activate_lobby(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            force_activation=force_activation if force_activation is not None else None,
            auth=auth,
        )
        return response["Response"]

    async def activate_lobby_for_new_listing_id(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        force_activation: Optional[bool] = None,
        auth: Optional[AuthData] = None,
    ) -> bool:
        """
        Activates a lobby and initializes it as an active Fireteam, returning the updated Listing ID.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to activate.
            force_activation: Optional boolean to forcibly activate the lobby, kicking pending applicants.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.activate_lobby_for_new_listing_id(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            force_activation=force_activation if force_activation is not None else None,
            auth=auth,
        )
        return response["Response"]

    async def apply_to_listing(
        self,
        application_type: Union[DestinyFireteamFinderApplicationType, int],
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        listing_id: int,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderApplyToListingResponse:
        """
        Applies to have a character join a fireteam.

        Args:
            application_type: The type of application to apply
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            listing_id: The id of the listing to apply to
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.apply_to_listing(
            application_type=getattr(application_type, "value", application_type),
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            listing_id=listing_id,
            auth=auth,
        )
        return await DestinyFireteamFinderApplyToListingResponse.from_dict(
            data=response,
            client=self._client,
            application_type=application_type,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            listing_id=listing_id,
            auth=auth,
        )

    async def bulk_get_listing_status(
        self,
        data: dict,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderBulkGetListingStatusResponse:
        """
        Retrieves Fireteam listing statuses in bulk.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.bulk_get_listing_status(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderBulkGetListingStatusResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def get_application(
        self,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetApplicationResponse:
        """
        Retrieves a Fireteam application.

        Args:
            application_id:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_application(
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
        )
        return await DestinyFireteamFinderGetApplicationResponse.from_dict(
            data=response,
            client=self._client,
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def get_listing(self, listing_id: int, auth: Optional[AuthData] = None) -> DestinyFireteamFinderListing:
        """
        Retrieves a Fireteam listing.

        Args:
            listing_id: The ID of the listing to retrieve.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_listing(listing_id=listing_id, auth=auth)
        return await DestinyFireteamFinderListing.from_dict(
            data=response, client=self._client, listing_id=listing_id, auth=auth
        )

    async def get_listing_applications(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        listing_id: int,
        flags: Optional[int] = None,
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetListingApplicationsResponse:
        """
        Retrieves all applications to a Fireteam Finder listing.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            listing_id: The ID of the listing whose applications to retrieve.
            flags: Optional flag representing a filter on the state of the application.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_listing_applications(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            listing_id=listing_id,
            flags=flags if flags is not None else None,
            next_page_token=next_page_token if next_page_token is not None else None,
            page_size=page_size if page_size is not None else None,
            auth=auth,
        )
        return await DestinyFireteamFinderGetListingApplicationsResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            listing_id=listing_id,
            flags=flags,
            next_page_token=next_page_token,
            page_size=page_size,
            auth=auth,
        )

    async def get_lobby(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderLobbyResponse:
        """
        Retrieves the information for a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to retrieve.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_lobby(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            auth=auth,
        )
        return await DestinyFireteamFinderLobbyResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            lobby_id=lobby_id,
            auth=auth,
        )

    async def get_player_lobbies(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetPlayerLobbiesResponse:
        """
        Retrieves the information for a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_player_lobbies(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            next_page_token=next_page_token if next_page_token is not None else None,
            page_size=page_size if page_size is not None else None,
            auth=auth,
        )
        return await DestinyFireteamFinderGetPlayerLobbiesResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            next_page_token=next_page_token,
            page_size=page_size,
            auth=auth,
        )

    async def get_player_applications(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetPlayerApplicationsResponse:
        """
        Retrieves Fireteam applications that this player has sent or recieved.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_player_applications(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            next_page_token=next_page_token if next_page_token is not None else None,
            page_size=page_size if page_size is not None else None,
            auth=auth,
        )
        return await DestinyFireteamFinderGetPlayerApplicationsResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            next_page_token=next_page_token,
            page_size=page_size,
            auth=auth,
        )

    async def get_player_offers(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetPlayerOffersResponse:
        """
        Retrieves Fireteam offers that this player has recieved.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_player_offers(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            next_page_token=next_page_token if next_page_token is not None else None,
            page_size=page_size if page_size is not None else None,
            auth=auth,
        )
        return await DestinyFireteamFinderGetPlayerOffersResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            next_page_token=next_page_token,
            page_size=page_size,
            auth=auth,
        )

    async def get_character_activity_access(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetCharacterActivityAccessResponse:
        """
        Retrieves the information for a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_character_activity_access(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
        )
        return await DestinyFireteamFinderGetCharacterActivityAccessResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def get_offer(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        offer_id: int,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderOffer:
        """
        Retrieves an offer to a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            offer_id: The unique ID of the offer.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_offer(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            offer_id=offer_id,
            auth=auth,
        )
        return await DestinyFireteamFinderOffer.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            offer_id=offer_id,
            auth=auth,
        )

    async def get_lobby_offers(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetLobbyOffersResponse:
        """
        Retrieves all offers relevant to a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The unique ID of the lobby.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_lobby_offers(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            next_page_token=next_page_token if next_page_token is not None else None,
            page_size=page_size if page_size is not None else None,
            auth=auth,
        )
        return await DestinyFireteamFinderGetLobbyOffersResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            lobby_id=lobby_id,
            next_page_token=next_page_token,
            page_size=page_size,
            auth=auth,
        )

    async def host_lobby(
        self,
        data: DestinyFireteamFinderHostLobbyRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderHostLobbyResponse:
        """
        Creates a new Fireteam lobby and Fireteam Finder listing.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.host_lobby(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderHostLobbyResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def join_lobby(
        self,
        data: DestinyFireteamFinderJoinLobbyRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderLobbyResponse:
        """
        Sends a request to join an available Fireteam lobby.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.join_lobby(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderLobbyResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def kick_player(
        self,
        data: DestinyFireteamFinderKickPlayerRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        target_membership_id: int,
        auth: Optional[AuthData] = None,
    ) -> bool:
        """
        Kicks a player from a Fireteam Finder lobby.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to kick the player from.
            target_membership_id: A valid Destiny membership ID of the player to kick.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.kick_player(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            target_membership_id=target_membership_id,
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return response["Response"]

    async def leave_application(
        self,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> bool:
        """
        Sends a request to leave a Fireteam listing application.

        Args:
            application_id: The ID of the application to leave.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.leave_application(
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
        )
        return response["Response"]

    async def leave_lobby(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        auth: Optional[AuthData] = None,
    ) -> bool:
        """
        Sends a request to leave a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to leave.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.leave_lobby(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            auth=auth,
        )
        return response["Response"]

    async def respond_to_application(
        self,
        data: DestinyFireteamFinderRespondToApplicationRequest,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderRespondToApplicationResponse:
        """
        Responds to an application sent to a Fireteam lobby.

        Args:
            data: The required data for this request.
            application_id: The application ID to respond to.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.respond_to_application(
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderRespondToApplicationResponse.from_dict(
            data=response,
            client=self._client,
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def respond_to_authentication(
        self,
        data: DestinyFireteamFinderRespondToAuthenticationRequest,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderRespondToAuthenticationResponse:
        """
        Responds to an authentication request for a Fireteam.

        Args:
            data: The required data for this request.
            application_id: The ID of the application whose authentication to confirm.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.respond_to_authentication(
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderRespondToAuthenticationResponse.from_dict(
            data=response,
            client=self._client,
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def respond_to_offer(
        self,
        data: DestinyFireteamFinderRespondToOfferRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        offer_id: int,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderRespondToOfferResponse:
        """
        Responds to a Fireteam lobby offer.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            offer_id: The unique ID of the offer.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.respond_to_offer(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            offer_id=offer_id,
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderRespondToOfferResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            offer_id=offer_id,
            auth=auth,
        )

    async def search_listings_by_clan(
        self,
        data: DestinyFireteamFinderSearchListingsByClanRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderSearchListingsByClanResponse:
        """
        Returns search results for available Fireteams provided a clan.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.search_listings_by_clan(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderSearchListingsByClanResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def search_listings_by_filters(
        self,
        data: DestinyFireteamFinderSearchListingsByFiltersRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        override_offline_filter: Optional[bool] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderSearchListingsByFiltersResponse:
        """
        Returns search results for available Fireteams provided search filters.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            override_offline_filter: Optional boolean to bypass the offline-only check, so the client can pull fireteam from the game.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.search_listings_by_filters(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            override_offline_filter=override_offline_filter if override_offline_filter is not None else None,
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderSearchListingsByFiltersResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            override_offline_filter=override_offline_filter,
            auth=auth,
        )

    async def update_lobby_settings(
        self,
        data: DestinyFireteamFinderUpdateLobbySettingsRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderUpdateLobbySettingsResponse:
        """
        Updates the settings for a Fireteam lobby.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to update.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.update_lobby_settings(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderUpdateLobbySettingsResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            lobby_id=lobby_id,
            auth=auth,
        )



from typing import TYPE_CHECKING, Optional, Union

from bungio.models.base import ClientMixin, FuzzyAttrFinder, custom_define

if TYPE_CHECKING:
    # AUTOMATIC IMPORTS START
    from bungio.models import (
        BungieMembershipType,
        ClanBanner,
        DestinyClanAggregateStat,
        DestinyLeaderboard,
        DestinyMilestone,
        EntityActionResult,
        FireteamDateRange,
        FireteamPlatform,
        FireteamPublicSearchOption,
        FireteamResponse,
        FireteamSlotSearch,
        GroupApplicationListRequest,
        GroupApplicationRequest,
        GroupEditAction,
        GroupOptionalConversation,
        GroupOptionalConversationAddRequest,
        GroupOptionalConversationEditRequest,
        GroupOptionsEditAction,
        GroupResponse,
        RuntimeGroupMemberType,
        SearchResultOfFireteamResponse,
        SearchResultOfFireteamSummary,
        SearchResultOfGroupBan,
        SearchResultOfGroupEditHistory,
        SearchResultOfGroupMember,
        SearchResultOfGroupMemberApplication,
    )

    # AUTOMATIC IMPORTS END
    from bungio.models.auth import AuthData

__all__ = ("DestinyClanMixin",)


@custom_define()
class DestinyClanMixin(ClientMixin, FuzzyAttrFinder):
    # DO NOT CHANGE ANY CODE BELOW. Automatically generated and overwritten

    async def get_group(self, auth: Optional["AuthData"] = None) -> "GroupResponse":
        """
        Get information about a specific group of the given ID.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_group(group_id=self._fuzzy_getattr("group_id"), auth=auth)

    async def get_group_optional_conversations(
        self, auth: Optional["AuthData"] = None
    ) -> list["GroupOptionalConversation"]:
        """
        Gets a list of available optional conversation channels and their settings.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_group_optional_conversations(
            group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def edit_group(self, data: "GroupEditAction", auth: "AuthData") -> int:
        """
        Edit an existing group. You must have suitable permissions in the group to perform this operation. This latest revision will only edit the fields you pass in - pass null for properties you want to leave unaltered.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.edit_group(data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth)

    async def edit_clan_banner(self, data: "ClanBanner", auth: "AuthData") -> int:
        """
        Edit an existing group's clan banner. You must have suitable permissions in the group to perform this operation. All fields are required.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.edit_clan_banner(data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth)

    async def edit_founder_options(self, data: "GroupOptionsEditAction", auth: "AuthData") -> int:
        """
        Edit group options only available to a founder. You must have suitable permissions in the group to perform this operation.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.edit_founder_options(
            data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def add_optional_conversation(self, data: "GroupOptionalConversationAddRequest", auth: "AuthData") -> int:
        """
        Add a new optional conversation/chat channel. Requires admin permissions to the group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.add_optional_conversation(
            data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def edit_optional_conversation(
        self, data: "GroupOptionalConversationEditRequest", conversation_id: int, auth: "AuthData"
    ) -> int:
        """
        Edit the settings of an optional conversation/chat channel. Requires admin permissions to the group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            conversation_id: Conversation Id of the channel being edited.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.edit_optional_conversation(
            data=data, conversation_id=conversation_id, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_members_of_group(
        self,
        currentpage: int,
        member_type: Union["RuntimeGroupMemberType", int],
        name_search: str,
        auth: Optional["AuthData"] = None,
    ) -> "SearchResultOfGroupMember":
        """
        Get the list of members in a given group.

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 items per page.
            member_type: Filter out other member types. Use None for all members.
            name_search: The name fragment upon which a search should be executed for members with matching display or unique names.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_members_of_group(
            currentpage=currentpage,
            group_id=self._fuzzy_getattr("group_id"),
            member_type=member_type,
            name_search=name_search,
            auth=auth,
        )

    async def get_admins_and_founder_of_group(
        self, currentpage: int, auth: Optional["AuthData"] = None
    ) -> "SearchResultOfGroupMember":
        """
        Get the list of members in a given group who are of admin level or higher.

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 items per page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_admins_and_founder_of_group(
            currentpage=currentpage, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_banned_members_of_group(self, currentpage: int, auth: "AuthData") -> "SearchResultOfGroupBan":
        """
        Get the list of banned members in a given group. Only accessible to group Admins and above. Not applicable to all groups. Check group features.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 entries.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_banned_members_of_group(
            currentpage=currentpage, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_group_edit_history(self, currentpage: int, auth: "AuthData") -> "SearchResultOfGroupEditHistory":
        """
        Get the list of edits made to a given group. Only accessible to group Admins and above.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 entries.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_group_edit_history(
            currentpage=currentpage, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def abdicate_foundership(
        self,
        founder_id_new: int,
        membership_type: Union["BungieMembershipType", int],
        auth: Optional["AuthData"] = None,
    ) -> bool:
        """
        An administrative method to allow the founder of a group or clan to give up their position to another admin permanently.

        Args:
            founder_id_new: The new founder for this group. Must already be a group admin.
            membership_type: Membership type of the provided founderIdNew.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.abdicate_foundership(
            founder_id_new=founder_id_new,
            group_id=self._fuzzy_getattr("group_id"),
            membership_type=membership_type,
            auth=auth,
        )

    async def get_pending_memberships(
        self, currentpage: int, auth: "AuthData"
    ) -> "SearchResultOfGroupMemberApplication":
        """
        Get the list of users who are awaiting a decision on their application to join a given group. Modified to include application info.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 items per page.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_pending_memberships(
            currentpage=currentpage, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_invited_individuals(
        self, currentpage: int, auth: "AuthData"
    ) -> "SearchResultOfGroupMemberApplication":
        """
        Get the list of users who have been invited into the group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 items per page.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_invited_individuals(
            currentpage=currentpage, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def approve_all_pending(
        self, data: "GroupApplicationRequest", auth: "AuthData"
    ) -> list["EntityActionResult"]:
        """
        Approve all of the pending users for the given group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.approve_all_pending(
            data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def deny_all_pending(self, data: "GroupApplicationRequest", auth: "AuthData") -> list["EntityActionResult"]:
        """
        Deny all of the pending users for the given group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.deny_all_pending(data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth)

    async def approve_pending_for_list(
        self, data: "GroupApplicationListRequest", auth: "AuthData"
    ) -> list["EntityActionResult"]:
        """
        Approve all of the pending users for the given group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.approve_pending_for_list(
            data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def deny_pending_for_list(
        self, data: "GroupApplicationListRequest", auth: "AuthData"
    ) -> list["EntityActionResult"]:
        """
        Deny all of the pending users for the given group that match the passed-in .

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.deny_pending_for_list(
            data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_clan_weekly_reward_state(self, auth: Optional["AuthData"] = None) -> "DestinyMilestone":
        """
        Returns information on the weekly clan rewards and if the clan has earned them or not. Note that this will always report rewards as not redeemed.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_clan_weekly_reward_state(group_id=self._fuzzy_getattr("group_id"), auth=auth)

    async def get_clan_leaderboards(
        self, maxtop: int, modes: str, statid: str, auth: Optional["AuthData"] = None
    ) -> dict[str, dict[str, "DestinyLeaderboard"]]:
        """
        Gets leaderboards with the signed in user's friends and the supplied destinyMembershipId as the focus. PREVIEW: This endpoint is still in beta, and may experience rough edges. The schema is in final form, but there may be bugs that prevent desirable operation.

        Args:
            maxtop: Maximum number of top players to return. Use a large number to get entire leaderboard.
            modes: List of game modes for which to get leaderboards. See the documentation for DestinyActivityModeType for valid values, and pass in string representation, comma delimited.
            statid: ID of stat to return rather than returning all Leaderboard stats.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_clan_leaderboards(
            group_id=self._fuzzy_getattr("group_id"), maxtop=maxtop, modes=modes, statid=statid, auth=auth
        )

    async def get_clan_aggregate_stats(
        self, modes: str, auth: Optional["AuthData"] = None
    ) -> list["DestinyClanAggregateStat"]:
        """
        Gets aggregated stats for a clan using the same categories as the clan leaderboards. PREVIEW: This endpoint is still in beta, and may experience rough edges. The schema is in final form, but there may be bugs that prevent desirable operation.

        Args:
            modes: List of game modes for which to get leaderboards. See the documentation for DestinyActivityModeType for valid values, and pass in string representation, comma delimited.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_clan_aggregate_stats(
            group_id=self._fuzzy_getattr("group_id"), modes=modes, auth=auth
        )

    async def get_active_private_clan_fireteam_count(self, auth: "AuthData") -> int:
        """
        Gets a count of all active non-public fireteams for the specified clan. Maximum value returned is 25.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_active_private_clan_fireteam_count(
            group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_available_clan_fireteams(
        self,
        activity_type: int,
        date_range: Union["FireteamDateRange", int],
        page: int,
        platform: Union["FireteamPlatform", int],
        public_only: Union["FireteamPublicSearchOption", int],
        slot_filter: Union["FireteamSlotSearch", int],
        auth: "AuthData",
        exclude_immediate: bool,
        lang_filter: str,
    ) -> "SearchResultOfFireteamSummary":
        """
        Gets a listing of all of this clan's fireteams that are have available slots. Caller is not checked for join criteria so caching is maximized.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            activity_type: The activity type to filter by.
            date_range: The date range to grab available fireteams.
            page: Zero based page
            platform: The platform filter.
            public_only: Determines public/private filtering.
            slot_filter: Filters based on available slots
            auth: Authentication information.
            exclude_immediate: If you wish the result to exclude immediate fireteams, set this to true. Immediate-only can be forced using the dateRange enum.
            lang_filter: An optional language filter.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_available_clan_fireteams(
            activity_type=activity_type,
            date_range=date_range,
            group_id=self._fuzzy_getattr("group_id"),
            page=page,
            platform=platform,
            public_only=public_only,
            slot_filter=slot_filter,
            auth=auth,
            exclude_immediate=exclude_immediate,
            lang_filter=lang_filter,
        )

    async def get_my_clan_fireteams(
        self,
        include_closed: bool,
        page: int,
        platform: Union["FireteamPlatform", int],
        auth: "AuthData",
        group_filter: bool,
        lang_filter: str,
    ) -> "SearchResultOfFireteamResponse":
        """
        Gets a listing of all fireteams that caller is an applicant, a member, or an alternate of.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            include_closed: If true, return fireteams that have been closed.
            page: Deprecated parameter, ignored.
            platform: The platform filter.
            auth: Authentication information.
            group_filter: If true, filter by clan. Otherwise, ignore the clan and show all of the user's fireteams.
            lang_filter: An optional language filter.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_my_clan_fireteams(
            group_id=self._fuzzy_getattr("group_id"),
            include_closed=include_closed,
            page=page,
            platform=platform,
            auth=auth,
            group_filter=group_filter,
            lang_filter=lang_filter,
        )

    async def get_clan_fireteam(self, fireteam_id: int, auth: "AuthData") -> "FireteamResponse":
        """
        Gets a specific fireteam.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            fireteam_id: The unique id of the fireteam.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_clan_fireteam(
            fireteam_id=fireteam_id, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )
